import {Pagination, PaginationItem, PaginationLink, Tooltip, Input} from "reactstrap";
import {FontAwesomeIcon} from "@fortawesome/react-fontawesome";
import {faFastBackward, faFastForward, faStepBackward, faStepForward} from "@fortawesome/free-solid-svg-icons";
import React from "react";
import {pageStyle, toolTipStyle} from "../styles";

class PaginationComponent extends React.Component {
    constructor(props) {
        super(props);
        this.state = {
            tooltipOpen: false,
            tooltipValue: ''
        }
    }

    setTooltip() {
        this.setState(prevState => {
            return ({tooltipOpen: !prevState.tooltipOpen})
        })
    }

    handleChange(e) {
        this.setState({tooltipValue: e.target.value})
    }

    render() {
        const {page, handleQuery, pageCount} = this.props;
        const {tooltipOpen} = this.state;
        return (
            <Pagination style={pageStyle}>

                <PaginationItem>
                    <PaginationLink onClick={() => handleQuery({start: 0})}>
                        <FontAwesomeIcon icon={faFastBackward}/>
                    </PaginationLink>
                </PaginationItem>


                <PaginationItem>
                    <PaginationLink previous onClick={() => handleQuery(page - 1 < 0 ? {start: 0} : {start: page - 1})}>
                        <FontAwesomeIcon icon={faStepBackward}/>
                    </PaginationLink>
                </PaginationItem>


                <PaginationItem active onClick={() => handleQuery({start: page})}>
                    <PaginationLink>{page + 1}</PaginationLink>
                </PaginationItem>

                {pageCount !== 0 ? (
                    <div style={{display: 'flex'}}>
                        <Tooltip
                            style={toolTipStyle}
                            placement="top"
                            isOpen={tooltipOpen}
                            target="ToolTip"
                        >
                            <Input
                                size={"sm"}
                                type="number"
                                onChange={(e) => this.handleChange(e)}
                                onKeyDown={(e) => {
                                    e.keyCode === 13 ? handleQuery({start: this.state.tooltipValue - 1}) : null
                                }}
                            />
                        </Tooltip>
                        <PaginationItem>
                            <PaginationLink style={{fontWeight: 800}} id={"ToolTip"} onClick={() => {
                                this.setTooltip()
                            }}>
                                . . .
                            </PaginationLink>
                        </PaginationItem>
                        <PaginationItem>
                            <PaginationLink
                                onClick={() => handleQuery({start: pageCount - 1})}>{pageCount}</PaginationLink>
                        </PaginationItem>
                    </div>
                ) : null}

                <PaginationItem>
                    <PaginationLink previous
                                    onClick={() => handleQuery(page + 1 > pageCount ? {start: pageCount} : {start: page + 1})}>
                        <FontAwesomeIcon icon={faStepForward}/>
                    </PaginationLink>
                </PaginationItem>

                <PaginationItem>
                    <PaginationLink onClick={() => handleQuery({start: pageCount - 1})}>
                        <FontAwesomeIcon icon={faFastForward}/>
                    </PaginationLink>
                </PaginationItem>

            </Pagination>
        )
    }
}

export {PaginationComponent}

