import React, {Component} from "react";
import Alert from "reactstrap/lib/Alert";
import {Button, Input} from "reactstrap";
import {map} from "lodash";
import {FontAwesomeIcon} from "@fortawesome/react-fontawesome";
import {
    faBackspace,
    faEdit,
    faPowerOff,
    faSortAmountUpAlt,
    faSortAmountDownAlt,
    faTimes
} from "@fortawesome/free-solid-svg-icons";
import ButtonDropdown from "reactstrap/lib/ButtonDropdown";
import DropdownToggle from "reactstrap/lib/DropdownToggle";
import DropdownMenu from "reactstrap/lib/DropdownMenu";
import DropdownItem from "reactstrap/lib/DropdownItem";
import PropTypes from "prop-types";
import {Col, Row} from "reactstrap";
import {
    errorStyle,
    buttonStyle,
    iconStyle,
    columnStyle,
    indexStyle,
    endColumnBody,
    outColumnBody,
    inColumnBody,
    endColumnHeader,
    outColumnHeader,
    inColumnHeader,
    noPadding
} from '../styles'

class TableComponent extends Component {
    static propTypes = {
        data: PropTypes.shape().isRequired,
        head: PropTypes.shape().isRequired,
        query: PropTypes.shape().isRequired,
        error: PropTypes.shape().isRequired,
        handleQuery: PropTypes.func.isRequired,
        onUpdate: PropTypes.func.isRequired,
        onDeactivate: PropTypes.func.isRequired,
        styles: PropTypes.shape({
            indexStyle: PropTypes.shape(),
            endColumnHeader: PropTypes.shape(),
            outColumnHeader: PropTypes.shape(),
            inColumnHeader: PropTypes.shape(),
            endColumnBody: PropTypes.shape(),
            outColumnBody: PropTypes.shape(),
            inColumnBody: PropTypes.shape(),
            tableFontSize: PropTypes.shape()
        })
    }
    static defaultProps = {
        styles: {
            indexStyle: indexStyle,
            endColumnHeader: endColumnHeader,
            outColumnHeader: outColumnHeader,
            inColumnHeader: inColumnHeader,
            endColumnBody: endColumnBody,
            outColumnBody: outColumnBody,
            inColumnBody: inColumnBody,
            tableFontSize: {fontSize: '0.8rem'}
        }
    }

    constructor(props) {
        super(props);
        this.state = {
            dropdownOpen: false,
            inputValue: ''
        }
    }

    shipQuery = (query) => {
        this.props.handleQuery(query)
    }

    handleChange(e) {
        this.setState({inputValue: e.target.value})
    }

    render() {
        const {data, head, query, error, styles} = this.props;
        const {dropdownOpen} = this.state

        return (
            <div>
                <Row style={{display: 'block'}}>
                    {error.anErrorOccurred &&
                    <Alert color="danger" style={errorStyle}>
                        {error.message}
                    </Alert>}
                </Row>
                <Row>
                    {this.props.data !== false && (
                        <div>
                            <div style={styles.outColumnHeader}>
                                <div className={'col-sm-1'} style={{maxWidth: '0.8rem'}}></div>
                                <div style={styles.inColumnHeader}>
                                    {map(data[0], (value, key) => {
                                        if (head[key]) {
                                            if (key === 'active') {
                                                return (
                                                    <Col style={{columnStyle, marginRight: '1rem'}}>
                                                        <Button
                                                            color="secondary"
                                                            onClick={() => {
                                                                this.shipQuery((query.search[0].active === 'S') ? ({
                                                                    start: 0,
                                                                    active: 'N'
                                                                }) : ({start: 0, active: 'S'}))
                                                            }}>
                                                            <FontAwesomeIcon icon={faPowerOff}
                                                                             style={{iconStyle, ...noPadding}}/>
                                                        </Button>
                                                    </Col>
                                                )
                                            } else {
                                                return (
                                                    <Col style={columnStyle}>
                                                        <Input type={key} name={key} id={key} placeholder={head[key]}
                                                               onChange={(e) => this.handleChange(e)}
                                                               onKeyDown={(e) => {
                                                                   e.keyCode === 13 ? this.shipQuery({
                                                                       fname: key,
                                                                       fvalue: this.state.inputValue,
                                                                       start: 0
                                                                   }) : null
                                                               }}
                                                        />
                                                    </Col>
                                                )
                                            }
                                        }
                                    })}
                                </div>
                                <div style={styles.endColumnHeader}>
                                    <ButtonDropdown style={{marginRight: '1rem'}} isOpen={dropdownOpen}
                                                    toggle={() => this.setState({dropdownOpen: !dropdownOpen})}>
                                        <Button id="caret" color="secondary" style={buttonStyle} onClick={() => {
                                            this.shipQuery(query.search[0].infoOrder === 'DESC' ? ({infoOrder: 'ASC'}) : ({infoOrder: 'DESC'}))
                                        }}>
                                            <FontAwesomeIcon
                                                icon={query.search[0].infoOrder === 'ASC' ? faSortAmountUpAlt : faSortAmountDownAlt}
                                                style={{iconStyle, ...noPadding}}/>
                                        </Button>
                                        <DropdownToggle split color="secondary" style={buttonStyle}/>
                                        <DropdownMenu>
                                            <DropdownItem header>Ordenar por:</DropdownItem>
                                            <DropdownItem divider/>
                                            {map(head, (value, key) => {
                                                return (
                                                    <DropdownItem
                                                        style={query.search[0].orderby === key ? ({backgroundColor: 'grey'}) : null}
                                                        onClick={() => {
                                                            this.shipQuery({orderby: key})
                                                        }}>
                                                        {value}
                                                    </DropdownItem>)
                                            })}
                                        </DropdownMenu>
                                    </ButtonDropdown>
                                    <Button color="secondary" onClick={() => {
                                        this.shipQuery({9999: '9999'})
                                    }}>
                                        <FontAwesomeIcon icon={faTimes} style={{iconStyle, ...noPadding}}/>
                                    </Button>
                                </div>
                            </div>
                            <div>
                                {map(data, (objj, key) => {
                                    return (
                                        <div
                                            style={{backgroundColor: (key % 2 === 0 ? 'lightgray' : 'white'), ...styles.outColumnBody}}>
                                            <Col className={'col-sm'} style={indexStyle}>
                                                {key + 1}
                                            </Col>
                                            <div style={styles.inColumnBody}>
                                                {map(objj, (obj, key) => {
                                                    if (head[key]) {
                                                        return <Col
                                                            style={{...styles.tableFontSize, ...noPadding}}>{obj}</Col>;
                                                    }
                                                })}
                                            </div>
                                            <Col className={'col-sm'} style={styles.endColumnBody}>
                                                <Button style={{...buttonStyle, marginRight: '2rem'}} color="danger"
                                                        onClick={() => {
                                                            this.props.onDeactivate(objj.id, objj.number, objj.active)
                                                            this.shipQuery({reload: 'reload'})
                                                        }}>
                                                    <FontAwesomeIcon icon={faBackspace} style={iconStyle}/>
                                                </Button>
                                                <Button style={buttonStyle} color="success" onClick={() => {
                                                    this.props.onUpdate(objj.id)
                                                    this.shipQuery({reload: 'reload'})
                                                }}>
                                                    <FontAwesomeIcon icon={faEdit} style={iconStyle}/>
                                                </Button>
                                            </Col>
                                        </div>
                                    );
                                })}
                            </div>
                        </div>)
                    }
                </Row>
            </div>
        );
    }
}

export {TableComponent}
